package fr.cnam.poker.client;

import java.io.*;
import java.net.*;

import fr.cnam.poker.commun.ReponseVoid;
import fr.cnam.poker.commun.RequeteNotification;
import fr.cnam.poker.commun.TablePoker;

// Classe de dfintion du serveur de socket utilis par l'IHM de Poker
//
// Cette classe est un SINGLETON*
//
public class ServeurIHM {
	
	// Dfinition du singleton
	private static ServeurIHM singleton=null;
	public static ServeurIHM getInstance() throws Exception {
		if (singleton==null)singleton=new ServeurIHM();
		return singleton;
	}
	
	private String        hostServeurPoker;   // Host du serveur de poker (pass en paramtre du main)  
	private int           portServeurPoker;   // Port du serveur depoker  (pass en paramtre du main)
	
	// Ces deux valeurs sont envoyes dans la requte d'abonnement au serveur de poker pour recevoir la notification.
	private String        hostServeurIHM;      // Host de la machine locale. 
	private int           portServeurIHM;      // Port du serveur de socket de l'IHM. 


	
	private ServeurIHM(){
		hostServeurPoker = "localhost";  // valeur dfaut
		portServeurPoker = 9100;         // valeur par dfaut
		hostServeurIHM="localhost";      // valeur par dfaut
		portServeurIHM = 9101;           // valeur par dfaut

	}
	
	// Mthode de dmarrage du serveur de socker
	//
	public void start() throws Exception {
		ServerSocket sos = new ServerSocket(portServeurIHM);  // Cration du serveur de socket
		while(true)
		{
			Socket soc = sos.accept();        // Attente de la connexion d'un socket
			InputStream is=soc.getInputStream();
			ObjectInputStream ois=new ObjectInputStream(is);
			OutputStream os=soc.getOutputStream();
			ObjectOutputStream oos=new ObjectOutputStream(os);

			Object requete = ois.readObject();    // Lecture de la requte reue
			
			System.out.println("REQUETE RECUE : "+requete.getClass().getName());

			//  Suivant le principe du Design Pattern MVC, la seule requte que peut recevoir l'IHM est
			//   la requte de notification contenant l'objet notifi par le modle de poker.
			//   Cet objet est une table de Poker qui contient tout l'tat dont a besoin l'IHM pour se rafraichir
			//
			if (requete instanceof RequeteNotification) {
				RequeteNotification req = (RequeteNotification)requete;
				TablePoker tablePoker=(TablePoker)req.getArg();
				
				// Mise  jour de l'IHM de Poker en fonction de l'objet notifi : la table de poker
				VuePokerIHM.getInstance().majTablePoker(tablePoker);   
				oos.writeObject(new ReponseVoid());  // Rponse void
			}
		}
	}
	
	
	public String getHostServeurPoker() {
		return hostServeurPoker;
	}

	public int getPortServeurPoker() {
		return portServeurPoker;
	}

	public void setHostServeurPoker(String hostServeurPoker) {
		this.hostServeurPoker = hostServeurPoker;
	}

	public void setPortServeurPoker(int portServeurPoker) {
		this.portServeurPoker = portServeurPoker;
	}
	
	public void setHostServeurIHM(String hostServeurIHM) {
		this.hostServeurIHM = hostServeurIHM;
	}

	public void setPortServeurIHM(int portServeurIHM) {
		this.portServeurIHM = portServeurIHM;
	}

	public String getHostServeurIHM() {
		return hostServeurIHM;
	}

	public int getPortServeurIHM() {
		return portServeurIHM;
	}


}
