package fr.cnam.poker.client;

import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Inet4Address;
import java.net.Socket;

import javax.swing.JLabel;

import fr.cnam.ihm.*;
import fr.cnam.poker.commun.*;

// Classe de dfinition d'une IHM de poker qui communique avec le serveur de Poker
// Cette classe est une VUE dans le modle MVC.
// 
// Cette IHM utilise la classe Formulaire pour crer l'interface homme-machine.
//
// Cette classe est un SINGLETON car l'IHM est unqiue dans la JVM d'un joueur
//
public class VuePokerIHM  implements FormulaireInt {

	// Dfintion du Singleton
	private static VuePokerIHM singleton = null;
	public static VuePokerIHM getInstance() {
		if (singleton==null)singleton=new VuePokerIHM();
		return singleton;
	}


	private int           numeroJoueur;       // Numro du joueur  (valeur donne par le serveur en rponse  la requte d'abonnement
	private String        nomJoueur;          // Nom du joueur (pass en paramtre du main)

	private Formulaire  form;                  // Le formulaire de l'IHM
	private JLabel cartes_joueurs[][];         // Les images des cartes de tous les joueurs
	private JLabel cartes_flop[];              // Les images des cartes du flop
	private JLabel carte_turn;                 // L'image de la carte du turn
	private JLabel carte_river;                // L'image de la carte de la river
	private JLabel courants[];                 // L'indicateur de chacun des joueurs pour marquer quel joueur est le courant (doit jouer)
	private JLabel donneurs[];                 // L'indicateur de chacun des joueurs pour marquer qui est le donneur.

	int widthCarte;           // Largeur d'une carte
	int heightCarte;          // Hauteur d'une carte
	int zoneJoueurX;          // Position en X de la zone des joueurs        
	int zoneJoueurY;          // Position en Y de la zone des joueurs
	int positionXFlop;
	int positionYFlop;

	// Constructeur de l'IHM d'un joueur
	//
	private VuePokerIHM() {
		// Attributs de connexion de l'IHM
		nomJoueur = "INCONNU";           // valeur dfaut

		numeroJoueur=-1; // Numro du joueur de l'IHM (par dfaut -1)

		// Cration du formulaire
		
		form =new Formulaire("JEU DE POKER",this,670,600,true);
		form.setTailleFonte(16);
		
		widthCarte=62;
		heightCarte=90;
		
		zoneJoueurX = 300;
		zoneJoueurY= 10;
		positionXFlop = 310;
		positionYFlop = 420;

		cartes_joueurs = new JLabel[4][2];
		courants = new JLabel[4];
		donneurs = new JLabel[4];

		// Les boutons du donneur
		
		form.setPosition(10,10);
		form.addText("NOM_JOUEUR", "", false, nomJoueur);
		form.addButton("COMMENCER", "Commencer");
		form.horizontal();
		form.addButton("FLOP","Flop");
		form.addButton("TURN","Turn");
		form.addButton("RIVER","River");
		form.dessous(10);
		form.addText("VALEUR_BLIND", "Blind", true, "");
		form.addButton("CHANGER_BLIND","Valider");
		
		// Les boutons de tous les joueurs
		form.dessous(10);
		form.vertical();
		form.setPosition(10,form.getYCour()+20);
		form.addButton("MISER_BLIND","Miser Blind");
		form.addButton("MISER_DOUBLE_BLIND","Miser Double Blind");
		form.addText("MISES", "Mises", true, "");
		form.addButton("MISER","Miser");
		form.addButton("CHECKER","Checker");
		form.addButton("FAIRE_TAPIS","ALL-IN");
		form.addButton("ABANDONNER","Abandonner");
		form.addButton("RAMASSER_POT","Rammasser le pot");
		form.addButton("MONTRER_CARTES","Montrer ses cartes");
		form.addButton("GAGNANT","Gagnant ?");
		form.addText("POT", "Pot : ", false,"");

		form.addZoneText("MESSAGE", "", true, "",200,80);
		form.addButton("ENVOYER_MESSAGE", "Envoyer");
		
		// La zone d'information visibles par tous les joueurs
		
		form.addZoneText ("INFOS", "",false,"",280,120);
		
		//  Les blocs de chacun des joueurs. Pas plus de 4 joueurs
		
		creerBlocJoueur(0,zoneJoueurX,zoneJoueurY);
		creerBlocJoueur(1,zoneJoueurX+200,zoneJoueurY);
		creerBlocJoueur(2,zoneJoueurX+200,zoneJoueurY+200);
		creerBlocJoueur(3,zoneJoueurX,zoneJoueurY+200);
		
		// La zone de flop, turn, river
		
		cartes_flop = new JLabel[3];

		form.setPosition(positionXFlop,positionYFlop);
		cartes_flop[0] = form.addImage("NONE", positionXFlop, positionYFlop, widthCarte, heightCarte, "images/cards_gif/vide.gif");
		cartes_flop[1] = form.addImage("NONE", positionXFlop+widthCarte, positionYFlop, widthCarte, heightCarte, "images/cards_gif/vide.gif");
		cartes_flop[2] = form.addImage("NONE", positionXFlop+widthCarte*2, positionYFlop, widthCarte, heightCarte, "images/cards_gif/vide.gif");
		carte_turn=form.addImage("NONE", positionXFlop+widthCarte*3, positionYFlop, widthCarte, heightCarte, "images/cards_gif/vide.gif");
		carte_river=form.addImage("NONE", positionXFlop+widthCarte*4, positionYFlop, widthCarte, heightCarte, "images/cards_gif/vide.gif");

		// Affichage de l'IHM
		form.afficher();
	}

	// Cration de tous les lments graphiques d'un joueur donn
	//
	private void creerBlocJoueur(int numeroJoueur,int positionXBloc,int positionYBloc) {
		int num = numeroJoueur+1;
		cartes_joueurs[numeroJoueur][0] = form.addImage("NONE", positionXBloc, positionYBloc, widthCarte, heightCarte, "images/cards_gif/vide.gif");
		cartes_joueurs[numeroJoueur][1] = form.addImage("NONE", positionXBloc+widthCarte, positionYBloc, widthCarte, heightCarte, "images/cards_gif/vide.gif");
		form.setPosition(positionXBloc, positionYBloc+heightCarte);
		form.addText("NOM_JOUEUR_"+num, "", false, "JOUEUR_"+num);
		form.addText("JETONS_JOUEUR_"+num, "Jetons", false, "0000");
		form.addText("JETONS_MISES_"+num, "Mises", false, "0000");
		donneurs[numeroJoueur] = form.addImage("NONE", positionXBloc+widthCarte*2+10, positionYBloc,30,30,"images/donneurOff.gif");
		courants[numeroJoueur] = form.addImage("NONE", positionXBloc+widthCarte*2+10, positionYBloc+30,30,30,"images/courantOff.gif");
	
	}

	// Abonnement de l'IHM au serveur de Poker
	//  Envoi de la requte d'abonnement
	//
	public void sabonner() throws Exception {
		ReponseRequeteAbonnement reponse = 
				(ReponseRequeteAbonnement)envoyerRequete(
						new RequeteAbonnement(ServeurIHM.getInstance().getHostServeurIHM(),
								              ServeurIHM.getInstance().getPortServeurIHM(),nomJoueur));
		numeroJoueur=reponse.getNumero();
		if (numeroJoueur==-1) throw new RuntimeException("Imposible de crer le joueur (pas plus de 4 joueurs)");
	}

	// Mthode qui s'ecute quand on ralise une action dans un des lments du Formulaire
	//
	public void submit(Formulaire form, String action) {
		System.out.println("SUBMIT> "+action);


		try {
			if (action.equals("COMMENCER")){
				envoyerRequete(new RequeteAction(numeroJoueur,"COMMENCER"));
			}
			if (action.equals("MISER")) {
				String v = form.getValeurChamp("MISES");
				if (controleSaisirEntier(v))
				{
					int mises = Integer.parseInt(v);					
					envoyerRequete(new RequeteAction(numeroJoueur,"MISER",""+mises));
				}
			}
			if (action.equals("CHANGER_BLIND")) {
				String v = form.getValeurChamp("VALEUR_BLIND");
				if (controleSaisirEntier(v))
				{
					int blind =  Integer.parseInt(v);
					envoyerRequete(new RequeteAction(numeroJoueur,"CHANGER_BLIND",""+blind));
				}
			}
			if (action.equals("MISER_BLIND")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"MISER_BLIND"));
			}
			if (action.equals("MISER_DOUBLE_BLIND")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"MISER_DOUBLE_BLIND"));
			}
			if (action.equals("FLOP")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"FLOP"));
			}
			if (action.equals("TURN")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"TURN"));
			}
			if (action.equals("RIVER")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"RIVER"));
			}
			if (action.equals("CHECKER")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"CHECKER"));
			}
			if (action.equals("ABANDONNER")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"ABANDONNER"));
			}
			if (action.equals("RAMASSER_POT")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"RAMASSER_POT"));
			}
			if (action.equals("FAIRE_TAPIS")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"FAIRE_TAPIS"));
			}
			if (action.equals("MONTRER_CARTES")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"MONTRER_CARTES"));
			}
			if (action.equals("GAGNANT")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"GAGNANT"));
			}
			if (action.equals("ENVOYER_MESSAGE")) {
				envoyerRequete(new RequeteAction(numeroJoueur,"ENVOYER_MESSAGE",form.getValeurChamp("MESSAGE")));
			}
		}catch(Exception ex) {ex.printStackTrace();}
	}

	// Teste si la saisie d'une valeur entire est correct
	//
	private boolean controleSaisirEntier(String v) {
		if (v=="") 
		{
			form.setValeurChamp("ERREUR", "Saisir une valeur");
			return false;	
		}
		else
			try {
				Integer.parseInt(v);
				return true;
			}catch(Exception ex) {form.setValeurChamp("ERREUR", "Saisir un entier");return false;}
	}

	// Mthode qui met  jour l'IHM en fonction de l'tat d'une table de poker.
	//  Cette table de poker est envoy par le serveur par le principe de notification
	//
	public void majTablePoker(TablePoker tablePoker) {
		
		// Maj des informations de chacun des joueurs et calcul du pot (somme des mises de chacun des joueurs) :
		//  le nom du joueur, ses jetons, ses mises, l'indicateur de Donneur
		//
		int pot=0;
		for(int j=0;j<tablePoker.getNbJoueur();j++) {
			Joueur joueur=tablePoker.getJoueur(j);  // Un des joueurs
			for(int k=0;k<2;k++) {
				if (joueur.getCartes()[k]==null) form.setImage(cartes_joueurs[j][k], "images/cards_gif/vide.gif");
				else if ( j==numeroJoueur || joueur.isCartes_visibles() ) form.setImage(cartes_joueurs[j][k], joueur.getCartes()[k].getImage());
				else form.setImage(cartes_joueurs[j][k], "images/cards_gif/dos.gif");
			}
			int num=j+1;
			form.setValeurChamp("NOM_JOUEUR_"+num, joueur.getNom());
			form.setValeurChamp("JETONS_JOUEUR_"+num, ""+joueur.getNbJetons());
			form.setValeurChamp("JETONS_MISES_"+num, ""+joueur.getNbJetonsMises());
			if ( (j==numeroJoueur) || joueur.isCartes_visibles() )
				if (joueur.getCombinaison()!=null)
					form.setValeurChamp("VALEUR_MAIN_"+num, Combinaison.MAINS[joueur.getCombinaison().getValeurMain()]);
				else form.setValeurChamp("VALEUR_MAIN_"+num, "");

			form.setImage(courants[j], "images/courantOff.gif");  // Par dfaut non courant

			if (joueur.isDonneur())
				form.setImage(donneurs[j], "images/donneurOn.gif");
			else 
				form.setImage(donneurs[j], "images/donneurOff.gif");

			pot=pot+joueur.getNbJetonsMises();
		}//Joueurs

		// Maj du nom du joueur
		form.setValeurChamp("NOM_JOUEUR",nomJoueur);

		// Maj de l'indicateur du joueur courant	
		form.setImage(courants[tablePoker.getNumeroJoueurCourant()], "images/courantOn.gif");
		
		// Maj de la valeur de la blind
		form.setValeurChamp("VALEUR_BLIND", ""+tablePoker.getBlind());

		// Maj de la valeur du pot
		form.setValeurChamp("POT", ""+pot);

		// Maj des cartes du flop, du turn et de la river
		//
		if (tablePoker.getFlop()[0]!=null) form.setImage(cartes_flop[0], tablePoker.getFlop()[0].getImage());
		else form.setImage(cartes_flop[0], "images/cards_gif/vide.gif");
		if (tablePoker.getFlop()[1]!=null) form.setImage(cartes_flop[1], tablePoker.getFlop()[1].getImage());
		else form.setImage(cartes_flop[1], "images/cards_gif/vide.gif");
		if (tablePoker.getFlop()[2]!=null) form.setImage(cartes_flop[2], tablePoker.getFlop()[2].getImage());
		else form.setImage(cartes_flop[2], "images/cards_gif/vide.gif");
		if (tablePoker.getTurn()!=null) form.setImage(carte_turn, tablePoker.getTurn().getImage());
		else form.setImage(carte_turn, "images/cards_gif/vide.gif");
		if (tablePoker.getRiver()!=null) form.setImage(carte_river, tablePoker.getRiver().getImage());
		else form.setImage(carte_river, "images/cards_gif/vide.gif");
		
		// Maj des infos
		//
		form.setValeurChamp("INFOS", tablePoker.getInfos());

		// Activation et dsactivation des boutons
		//
		activerDesactiverActions(tablePoker);
	}
	
	// Mthode qui active et dsactive les boutons
	private void activerDesactiverActions(TablePoker tablePoker) {
		
		// Une fois la partie commence, tout joueur peut rammasser le pot ou montrer ses cartes
		if (tablePoker.isCommence()) {
			form.activer("RAMASSER_POT");
			form.activer("MONTRER_CARTES");
		}
		else {
			form.desactiver("RAMASSER_POT");
			form.desactiver("MONTRER_CARTES");
		}

		// S'il y a au moins 2 joueurs, que la partie a commence et que le joueur de l'IHM est
		//  le joueur courant alors il peut faire une des actions de jeu (miser, checker, abandonner, faire tapis)
		if((tablePoker.getNbJoueur()>=1)&&tablePoker.getNumeroJoueurCourant()==numeroJoueur&&tablePoker.isCommence()) {
			form.activer("MISER_BLIND");
			form.activer("MISER_DOUBLE_BLIND");
			form.activer("MISER");
			form.activer("CHECKER");
			form.activer("ABANDONNER");
			form.activer("FAIRE_TAPIS");
		}
		else {
			form.desactiver("MISER_BLIND");
			form.desactiver("MISER_DOUBLE_BLIND");
			form.desactiver("MISER");
			form.desactiver("CHECKER");
			form.desactiver("ABANDONNER");
			form.desactiver("FAIRE_TAPIS");
		}

		// Seul le donneur peut distribuer les cartes de flop, turn et river (si la partie a commenc)
		// Seul le donneur peut commencer une nouvelle manche et changer la valeur de la blind
		if ((tablePoker.getNbJoueur()>=2)&&tablePoker.getJoueur(numeroJoueur).isDonneur())
		{
			if(tablePoker.isCommence()) {
				form.activer("FLOP");
				form.activer("TURN");
				form.activer("RIVER");
			}
			form.activer("COMMENCER");
			form.activer("CHANGER_BLIND");
		}
		else {
			form.desactiver("FLOP");
			form.desactiver("TURN");
			form.desactiver("RIVER");
			form.desactiver("COMMENCER");
			form.desactiver("CHANGER_BLIND");
		}
	}

	// Mthode elmentaire d'envoi d'une requte au serveur de Poker
	//  La requte est envoy et on attend une rponse. La requte est un Object Java
	//  La mthode retourne la rponse qui est un Object Java.
	//
	public Object envoyerRequete(Object obj)  {
		try {
			Socket soc = new Socket(Inet4Address.getByName(ServeurIHM.getInstance().getHostServeurPoker()),
					                ServeurIHM.getInstance().getPortServeurPoker());
			OutputStream os=soc.getOutputStream();
			ObjectOutputStream oos=new ObjectOutputStream(os);
			InputStream is=soc.getInputStream();
			ObjectInputStream ois=new ObjectInputStream(is);

			oos.writeObject(obj);              // Ecriture de la requte
			Object retour=ois.readObject();    // Attente de la rponse
			soc.close();                       // Fermeture du socket
			return retour;
		}catch(Exception ex) {ex.printStackTrace();return null;}
	}


	public void setNomJoueur(String nomJoueur) {
		this.nomJoueur = nomJoueur;
	}


}
