package fr.cnam.poker.commun;

import java.io.Serializable;
import java.util.*;

// Classe de dfintion d'une combinaison de plusieurs cartes (5, 6 ou 7 cartes pour le poker).
// 
//  calculerMainDe5Cartes         : Cette mthode s'applique sur une combinaison de 5 cartes.
//                                  Elle permet de dterminer quel est la main de poker contenue dans ces 5 cartes 
//                                    (paire, double paire, brelan, ....)
//
//  calculerMainDe5cartesParmiCartes : Mthode static qui permet de dterminer la meilleure main de poker qu'il est possible
//                                     de faire parmis plusieurs cartes (par exemple 7 au poker (flop+turn+river)
//
//   comparer2Combinaisons   : Mthode sattic qui permet de comparer 2 combinaisons de 5 cartes afin de dtermine quelle
//                             combinaison contient la meileure main de poker
//
//
public class Combinaison implements Serializable {
	private static final long serialVersionUID = 1L;

	public static int NONE=0;
	public static int PAIRE=1;
	public static int DOUBLE_PAIRE=2;
	public static int BRELAN=3;
	public static int SUITE=4;
	public static int COULEUR=5;
	public static int FULL=6;
	public static int CARRE=7;
	public static int FLUSH=8;
	public static String[] MAINS = {"NONE","PAIRE","DOUBLE PAIRE","BRELAN","SUITE","COULEUR","FULL","CARRE","FLUSH"};

	private ArrayList<Carte> cartes;  // Les cartes de la combinaison (au poker jusqu' 7 cartes)
	private int valeurMain;
	private Carte[] paire1;
	private Carte[] paire2;
	private Carte[] brelan;
	private Carte[] carre;
	private Carte[] suite;
	private Carte[] couleur;

	public Combinaison(Carte c1,Carte c2,Carte c3,Carte c4,Carte c5) {
		cartes= new ArrayList<Carte>();
		cartes.add(c1);
		cartes.add(c2);
		cartes.add(c3);
		cartes.add(c4);
		cartes.add(c5);
		Collections.sort(cartes);
		paire1=new Carte[2];
		paire2=new Carte[2];
		brelan=new Carte[3];
		carre=new Carte[4];
		suite=new Carte[5];
		couleur=new Carte[5];
	}

	public int calculerMainDe5Cartes() {
		if (isFlush()) return Combinaison.FLUSH;
		else if (isCarre()) return Combinaison.CARRE;
		else if (isFull()) return Combinaison.FULL;
		else if (isCouleur()) return Combinaison.COULEUR;
		else if (isSuite()) return Combinaison.SUITE;
		else if (isBrelan()) return Combinaison.BRELAN;
		else if (isDoublePaire()) return Combinaison.DOUBLE_PAIRE;
		else if(isPaire()) return Combinaison.PAIRE;
		else return Combinaison.NONE;
	}

	public static Combinaison calculerMainDe5cartesParmiCartes(ArrayList<Carte> cartes) {

		Combinaison[] courantes=new Combinaison[9];  //Combinaison trouve pour chaque main

		Carte[] tabCartes = new Carte[cartes.size()];
		cartes.toArray(tabCartes);
		Combiner<Carte> combiner = new Combiner<Carte>(5,tabCartes);
		Carte[] combinaison = new Carte[5];
		int max=0;
		while(combiner.searchNext(combinaison))
		{
			Combinaison c = new Combinaison(combinaison[0],combinaison[1],combinaison[2],combinaison[3],combinaison[4]);
			int v = c.calculerMainDe5Cartes();
			if (v>=max) {
				max=v;
				if (courantes[max]==null)
					courantes[max]=c;	
				else {
					int res = comparer2Combinaisons(c,courantes[max]);
					if (res==1)courantes[max]=c;
				}
			}
		}
		courantes[max].valeurMain= max;
		return courantes[max];
	}
	
	static public int comparer2Combinaisons(Combinaison comb1,Combinaison comb2) {
		if (comb1.valeurMain<comb2.valeurMain) return -1;
		if (comb1.valeurMain>comb2.valeurMain) return 1;
		int val=comb1.valeurMain;
		if (val==Combinaison.NONE) {
			for(int i=4;i>=0;i--) {
				if (comb1.cartes.get(i).getRang()>comb2.cartes.get(i).getRang()) return 1;
				else if (comb1.cartes.get(i).getRang()<comb2.cartes.get(i).getRang()) return -1;
			}
			return 0;
		}
		else if (val==Combinaison.PAIRE) {
			if (comb1.paire1[0].getRang()>comb2.paire1[0].getRang()) return 1;
			else if (comb1.paire1[0].getRang()<comb2.paire1[0].getRang()) return -1;
			return comparerCartesRestantes(comb1.cartesRestantes(comb1.paire1),comb2.cartesRestantes(comb2.paire1));
		}
		else if (val==Combinaison.DOUBLE_PAIRE) {
			if (comb1.paire2[0].getRang()>comb2.paire2[0].getRang()) return 1;
			else if (comb1.paire2[0].getRang()<comb2.paire2[0].getRang()) return -1;
			else if (comb1.paire1[0].getRang()>comb2.paire1[0].getRang()) return 1;
			else if (comb1.paire1[0].getRang()<comb2.paire1[0].getRang()) return -1;
			else return comparerCartesRestantes(comb1.cartesRestantes(concat(comb1.paire1,comb1.paire2)),
					                            comb2.cartesRestantes(concat(comb2.paire1,comb2.paire2)));
		}
		else if (val==Combinaison.BRELAN) {
			if (comb1.brelan[0].getRang()>comb2.brelan[0].getRang()) return 1;
			else if (comb1.brelan[0].getRang()<comb2.brelan[0].getRang()) return -1;
			else return comparerCartesRestantes(comb1.cartesRestantes(comb1.brelan),
					                            comb2.cartesRestantes(comb2.brelan));
		}
		else if (val==Combinaison.SUITE) {
			for(int i=4;i>=0;i--)
			{
				if (comb1.suite[i].getRang()>comb2.suite[i].getRang()) return 1;
				else if (comb1.suite[i].getRang()<comb2.suite[i].getRang()) return -1;
			}
			return 0;
		}
		else if (val==Combinaison.COULEUR) {
			for(int i=4;i>=0;i--)
			{
				if (comb1.couleur[i].getRang()>comb2.couleur[i].getRang()) return 1;
				else if (comb1.couleur[i].getRang()<comb2.couleur[i].getRang()) return -1;
			}
			return 0;
		}
		else if (val==Combinaison.FULL) {
			if (comb1.brelan[0].getRang()>comb2.brelan[0].getRang()) return 1;
			else if (comb1.brelan[0].getRang()<comb2.brelan[0].getRang()) return -1;
			else if (comb1.paire1[0].getRang()>comb2.paire1[0].getRang()) return 1;
			else if (comb1.paire1[0].getRang()<comb2.paire1[0].getRang()) return -1;
			else return 0;
		}
		else if (val==Combinaison.CARRE) {
			if (comb1.carre[0].getRang()>comb2.carre[0].getRang()) return 1;
			else if (comb1.carre[0].getRang()<comb2.carre[0].getRang()) return -1;
			else return comparerCartesRestantes(comb1.cartesRestantes(comb1.carre),
					                            comb2.cartesRestantes(comb2.carre));
		}
		else if (val==Combinaison.FLUSH) {
			for(int i=4;i>=0;i--) {
				if (comb1.suite[i].getRang()>comb2.suite[i].getRang()) return 1;
				else if (comb1.suite[i].getRang()<comb2.suite[i].getRang()) return -1;
			}
			return 0;
		}
		return 0; // Jamais
	}
	
	public ArrayList<Carte> cartesRestantes(Carte[] main) {
		ArrayList<Carte> restantes = new ArrayList<Carte>();
		for(int i=4;i>=0;i--) {
			boolean trouve=false;
			for(Carte c:main) if(c==cartes.get(i))trouve=true;
			if (!trouve) {restantes.add(cartes.get(i));
			}
		}
		return restantes;
	}
	
	static public int comparerCartesRestantes(ArrayList<Carte> carte1,ArrayList<Carte> carte2) {
		System.out.println("comparerCartesRestantes");
		for(int i=carte1.size()-1;i>=0;i--) System.out.print(carte1.get(i));System.out.println();
		for(int i=carte2.size()-1;i>=0;i--) System.out.print(carte2.get(i));System.out.println();
		for(int i=carte1.size()-1;i>=0;i--) {
			if (carte1.get(i).getRang()>carte2.get(i).getRang()) return 1;
			else if (carte1.get(i).getRang()<carte2.get(i).getRang()) return -1;
		}
		return 0;
	}
	
	private static Carte[] concat(Carte[] carte1,Carte[] carte2) {
		Carte[] cartes=new Carte[carte1.length+carte2.length];
		int k=0;
		for(int i=0;i<carte1.length;i++) {cartes[k]=carte1[i];k++;}
		for(int i=0;i<carte2.length;i++) {cartes[k]=carte2[i];k++;}
		return cartes;
	}

	public boolean isCouleur() {
		int n=0;
		for(Carte c:cartes)
			if (c.getCouleur().equals(cartes.get(0).getCouleur())) {
				couleur[n]=c;
				n++;
			}
		if (n==5) return true;
		else return false;
	}

	public boolean isSuite() {
		int n=0;
		for(int i=0;i<4;i++) {
			if (cartes.get(i).getRang()==cartes.get(i+1).getRang()-1) {
				suite[n]=cartes.get(i);
				n++;
			}
		}
		if (n==4) {
			suite[4]=cartes.get(4);
			return true;
		}
		else return false;
	}

	public boolean isFlush() {
		return(isSuite()&&isCouleur());
	}

	public boolean isCarre() {
		int n=0;
		for(int i=0;i<3;i++) {
			if (cartes.get(i).getRang()==cartes.get(i+1).getRang()) {
				carre[n]=cartes.get(i);
				n++;
			}
		}
		if (n==3) {
			carre[3]=cartes.get(3);
			return true;
		}
		else {
			n=0;
			for(int i=1;i<4;i++) {
				if (cartes.get(i).getRang()==cartes.get(i+1).getRang()) {
					carre[n]=cartes.get(i);
					n++;
				}
			}
			if (n==3) {
				carre[3]=cartes.get(4);
				return true;
			}
			else return false;
		}
	}

	public boolean isBrelan() {
		//if (isCarre()) return false;
		//if (isFull()) return false;
		int n=0;
		for(int i=0;i<2;i++) {
			if (cartes.get(i).getRang()==cartes.get(i+1).getRang()) {
				brelan[n]=cartes.get(i);
				n++;
			}
		}
		if (n==2) {
			brelan[2]=cartes.get(2);
			return true;
		}
		else {
			n=0;
			for(int i=1;i<3;i++) {
				if (cartes.get(i).getRang()==cartes.get(i+1).getRang()) {
					brelan[n]=cartes.get(i);
					n++;
				}
			}
			if (n==2) {
				brelan[2]=cartes.get(3);
				return true;
			}
			else {
				n=0;
				for(int i=2;i<4;i++) {
					if (cartes.get(i).getRang()==cartes.get(i+1).getRang()) {
						brelan[n]=cartes.get(i);
						n++;
					}
				}
				if (n==2) {
					brelan[2]=cartes.get(4);
					return true;
				}
				else return false;
			}
		}
	}

	public boolean isDoublePaire() {
		//return (nbPaires()==2)&&(!isBrelan());
		return (nbPaires()==2);
	}

	public boolean isPaire() {
		return (nbPaires()==1);
	}

	public boolean isFull() {
		//return (nbPaires()==2)&&(!isCarre());
		//return (nbPaires()==1)&&(isBrelan());
		if ( (cartes.get(0).getRang()==cartes.get(1).getRang())&&(cartes.get(2).getRang()==cartes.get(3).getRang())&&(cartes.get(3).getRang()==cartes.get(4).getRang()) ) {
			paire1[0]=cartes.get(0);
			paire1[1]=cartes.get(1);
			brelan[0]=cartes.get(2);
			brelan[1]=cartes.get(3);
			brelan[2]=cartes.get(4);
			return true;
		}
		else if ( (cartes.get(0).getRang()==cartes.get(1).getRang())&&(cartes.get(1).getRang()==cartes.get(2).getRang())&&(cartes.get(3).getRang()==cartes.get(4).getRang()) ) {
			brelan[0]=cartes.get(0);
			brelan[1]=cartes.get(1);
			brelan[2]=cartes.get(2);
			paire1[0]=cartes.get(3);
			paire1[1]=cartes.get(4);
			return true;
		}
		else return false;
	}

	// Un carr retourne 3 paires
	// Un brelan retourne 2 paires
	// Un full retourne 3 paires
	//
	public int nbPaires() {
		int nbPaires=0;
		for(int i=0;i<4;i++) {
			if (cartes.get(i).getRang()==cartes.get(i+1).getRang()) {
				if (nbPaires==0) {
					paire1[0]=cartes.get(i);
					paire1[1]=cartes.get(i+1);
				}else if (nbPaires==1){
					paire2[0]=cartes.get(i);
					paire2[1]=cartes.get(i+1);
				}
				nbPaires++;
			}
		}
		return nbPaires;
	}

	public String toString() {
		String s="";
		s=s+MAINS[valeurMain]+"\n";
		for(Carte c:cartes)s=s+c+"/";s=s+"\n";
		if (valeurMain==PAIRE)for(Carte c:paire1)s=s+c+"/";
		if (valeurMain==DOUBLE_PAIRE) {for(Carte c:paire1)s=s+c+"/";for(Carte c:paire2)s=s+c+"/";}
		if (valeurMain==BRELAN)for(Carte c:brelan)s=s+c+"/";
		if (valeurMain==SUITE)for(Carte c:suite)s=s+c+"/";
		if (valeurMain==COULEUR)for(Carte c:couleur)s=s+c+"/";
		if (valeurMain==FULL) {for(Carte c:brelan)s=s+c+"/";for(Carte c:paire1)s=s+c+"/";}
		if (valeurMain==CARRE)for(Carte c:carre)s=s+c+"/";
		if (valeurMain==FLUSH)for(Carte c:suite)s=s+c+"/";
		return s;
	}
	

	public ArrayList<Carte> getCartes() {
		return cartes;
	}

	public int getValeurMain() {
		return valeurMain;
	}

	public Carte[] getPaire1() {
		return paire1;
	}

	public Carte[] getPaire2() {
		return paire2;
	}

	public Carte[] getBrelan() {
		return brelan;
	}

	public Carte[] getCarre() {
		return carre;
	}

	public Carte[] getSuite() {
		return suite;
	}

	public Carte[] getCouleur() {
		return couleur;
	}



	// ======================================================================
	//
	public static class Combiner<T>
	{
		protected int count;
		protected T[] array;
		protected int[] indexes;

		public Combiner( int count, T[] array )
		{
			super();
			this.count = count;
			this.array = array;
			indexes = new int[count];
			for ( int i = 0; i < count; i++ )
				indexes[i] = i;
		}

		public boolean searchNext( T[] result )
		{
			if ( indexes == null )
				return false;

			int resultIndex = 0;
			for ( int index : indexes )
				result[resultIndex++] = array[index];

			int indexesRank = count-1;
			int arrayRank = array.length-1;
			while ( indexes[indexesRank] == arrayRank )
			{
				if ( indexesRank == 0 )
				{
					indexes = null;
					return true;
				}
				indexesRank--;
				arrayRank--;
			}

			int restartIndex = indexes[indexesRank] + 1;
			while ( indexesRank < count )
				indexes[indexesRank++] = restartIndex++;

			return true;
		}
	}

	// ===================================================================
	//     PROGRAMME DE TEST
	// ===================================================================
	public static void main(String... args) {

		ArrayList<Carte> cartes;
		// Tester la suite
		//
		System.out.println("SUITE>");
		cartes = new ArrayList<Carte>();
		cartes.add(new Carte(Carte.TREFLE,"2"));
		cartes.add(new Carte(Carte.TREFLE,"5"));
		cartes.add(new Carte(Carte.TREFLE,"4"));
		cartes.add(new Carte(Carte.TREFLE,"3"));
		cartes.add(new Carte(Carte.TREFLE,"6"));
		Combinaison comb = calculerMainDe5cartesParmiCartes(cartes);
		System.out.println(comb);
		
		
		// Tester le Carr
		//
		System.out.println("CARRE>");
		cartes = new ArrayList<Carte>();
		cartes.add(new Carte(Carte.TREFLE,"2"));
		cartes.add(new Carte(Carte.CARREAU,"5"));
		cartes.add(new Carte(Carte.PIQUE,"2"));
		cartes.add(new Carte(Carte.TREFLE,"2"));
		cartes.add(new Carte(Carte.COEUR,"2"));
		comb = calculerMainDe5cartesParmiCartes(cartes);
		System.out.println(comb);
		
		cartes = new ArrayList<Carte>();
		cartes.add(new Carte(Carte.TREFLE,"10"));
		cartes.add(new Carte(Carte.CARREAU,"5"));
		cartes.add(new Carte(Carte.PIQUE,"10"));
		cartes.add(new Carte(Carte.TREFLE,"10"));
		cartes.add(new Carte(Carte.COEUR,"10"));
		comb = calculerMainDe5cartesParmiCartes(cartes);
		System.out.println(comb);
		
		// Tester le Brelan
		//
		System.out.println("BRELAN>");
		cartes = new ArrayList<Carte>();
		cartes.add(new Carte(Carte.TREFLE,"2"));
		cartes.add(new Carte(Carte.CARREAU,"5"));
		cartes.add(new Carte(Carte.PIQUE,"2"));
		cartes.add(new Carte(Carte.TREFLE,"2"));
		cartes.add(new Carte(Carte.COEUR,"10"));
		comb = calculerMainDe5cartesParmiCartes(cartes);
		System.out.println(comb);
		
		cartes = new ArrayList<Carte>();
		cartes.add(new Carte(Carte.TREFLE,"10"));
		cartes.add(new Carte(Carte.CARREAU,"5"));
		cartes.add(new Carte(Carte.PIQUE,"10"));
		cartes.add(new Carte(Carte.TREFLE,"10"));
		cartes.add(new Carte(Carte.COEUR,Carte.ROI));
		comb = calculerMainDe5cartesParmiCartes(cartes);
		System.out.println(comb);

		// Tester la  paire
		//	
		System.out.println("PAIRE>");
		cartes = new ArrayList<Carte>();
		cartes.add(new Carte(Carte.TREFLE,"2"));
		cartes.add(new Carte(Carte.CARREAU,"5"));
		cartes.add(new Carte(Carte.PIQUE,"2"));
		cartes.add(new Carte(Carte.TREFLE,"10"));
		cartes.add(new Carte(Carte.COEUR,"7"));
		comb = calculerMainDe5cartesParmiCartes(cartes);
		System.out.println(comb);

		// Tester la double paire
		//		
		System.out.println("DOUBLE PAIRE>");
		cartes = new ArrayList<Carte>();
		cartes.add(new Carte(Carte.TREFLE,"2"));
		cartes.add(new Carte(Carte.CARREAU,"5"));
		cartes.add(new Carte(Carte.PIQUE,"2"));
		cartes.add(new Carte(Carte.TREFLE,"7"));
		cartes.add(new Carte(Carte.COEUR,"5"));
		comb = calculerMainDe5cartesParmiCartes(cartes);
		System.out.println(comb);

		// Tester sur 7 cartes
		System.out.println("7 CARTES>>");
		cartes = new ArrayList<Carte>();
		cartes.add(new Carte(Carte.TREFLE,"2"));
		cartes.add(new Carte(Carte.TREFLE,"2"));
		cartes.add(new Carte(Carte.TREFLE,"4"));
		cartes.add(new Carte(Carte.CARREAU,"4"));
		cartes.add(new Carte(Carte.TREFLE,"4"));
		cartes.add(new Carte(Carte.COEUR,"2"));
		cartes.add(new Carte(Carte.PIQUE,"4"));
		comb = calculerMainDe5cartesParmiCartes(cartes);
		System.out.println(comb);
		
		//Tester Comparaison
		ArrayList<Carte> cartes1 = new ArrayList<Carte>();
		cartes1.add(new Carte(Carte.PIQUE,Carte.ROI));
		cartes1.add(new Carte(Carte.PIQUE,Carte.DAME));
		cartes1.add(new Carte(Carte.CARREAU,Carte.AS));
		cartes1.add(new Carte(Carte.PIQUE,Carte.AS));
		cartes1.add(new Carte(Carte.TREFLE,Carte.AS));
		ArrayList<Carte> cartes2 = new ArrayList<Carte>();
		cartes2.add(new Carte(Carte.PIQUE,Carte.ROI));
		cartes2.add(new Carte(Carte.CARREAU,Carte.AS));
		cartes2.add(new Carte(Carte.PIQUE,Carte.AS));
		cartes2.add(new Carte(Carte.PIQUE,Carte.DAME));
		cartes2.add(new Carte(Carte.CARREAU,Carte.DAME));
		Combinaison comb1 = calculerMainDe5cartesParmiCartes(cartes1);
		Combinaison comb2 = calculerMainDe5cartesParmiCartes(cartes2);
		System.out.print(MAINS[comb1.getValeurMain()]+" ");
		System.out.println(MAINS[comb2.getValeurMain()]);
		int res = comparer2Combinaisons(comb1, comb2);
		if (res==-1) System.out.println("COMB 1 < COMB 2");
		else if (res==1) System.out.println("COMB 1 > COMB 2");
		else System.out.println("COMB 1 = COMB 2");
	}

}
