package fr.cnam.poker.commun;

import java.io.Serializable;

// Classe de dfinition d'un joueur
//   Un joueur est ajout dans TablePoker qui est donc notifi aux vues.
//
public class Joueur implements Serializable {
	private static final long serialVersionUID = 1L;
	
	private int       numero;           // de 0  N (indice dans le tableau de joueur dans TablePoker)
	private String    nom;              // Nom du joueur
	private Carte     cartes[];         // Les 2 cartes du joueur
	private int       nbJetons;         // Nombre de jeton en possession du joueur
	private int       nbJetonsMises;    // Nombre de jeton miss par un joueur durant une manche
	private boolean   donneur;          // Vrai si le joueur est le donneur (dans une manche)
	private boolean   abandon;          // Vrai si le donneur a abandonn (dans une manche)
	private boolean   tapis;            // Vrai si le joueur a fait tapis (dans une manche)
	private boolean   cartesVisibles;  // Vrai si le joueur a montr ses cartes (dans une manche)
	// Hors sujet
	private boolean robot;              // Vrai si le joueur est un robot
	private Combinaison combinaison;    // La combinaison la plus haute qu'un joueur peut faire avec ses cartes
									    // et les cartes du flop, du turn et de la river

	public Joueur() {
		super();
		this.numero=-1;
		this.nom = "INCONNU";
		this.cartes = new Carte[2];
		this.nbJetons = 1000;
		this.nbJetonsMises = 0;
		this.donneur = false;
		this.abandon = true;
		this.tapis   = false;
		this.cartesVisibles = false;
		this.robot = false;
		this.combinaison= null;
	}
	
	// Initialiser le joueur en dbut d'une nouvelle manche
	//
	public void initManche() {
		setAbandon(false);
		setTapis(false);
		setNbJetonsMises(0);
		setCombinaison(null);
		cartesVisibles=false;
	}
	
	// Miser des jetons  partir des jetons que le joueur possde
	//
	public void miser(int mises) {
		int jetons = nbJetons;
		if (jetons-mises<0) mises=jetons;
		int jetonsMises = nbJetonsMises;
		nbJetons = jetons-mises;
		nbJetonsMises=jetonsMises+mises;
	}
	
	// Ajouter des jetons aux jetons que le joueur possde
	//
	public void ajouterJetons(int jetons) {
		nbJetons=nbJetons+jetons;
	}
	
	// Enlever des jetons de ceux que le joueur posssde
	//
	public void enleverJetons(int jetons) {
		nbJetons=nbJetons-jetons;
	}
	
	// Retourne tous les jetons du joueur : ceux qu'il possde et ceux qu'il a miss
	//
	public int tousLesJetons() {
		return nbJetons+nbJetonsMises;
	}
	
	public String getNom() {
		return nom;
	}
	public void setNom(String nom) {
		this.nom = nom;
	}
	public Carte[] getCartes() {
		return cartes;
	}

	public int getNbJetons() {
		return nbJetons;
	}
	public void setNbJetons(int nbJetons) {
		this.nbJetons = nbJetons;
	}
	public int getNbJetonsMises() {
		return nbJetonsMises;
	}
	public void setNbJetonsMises(int nbJetonsMises) {
		this.nbJetonsMises = nbJetonsMises;
	}
	public void setDonneur(boolean donneur) {
		this.donneur = donneur;
	}
	
	public boolean isDonneur() {
		return donneur;
	}

	public boolean isAbandon() {
		return abandon;
	}
	public void setAbandon(boolean abandon) {
		this.abandon = abandon;
	}

	public boolean isTapis() {
		return tapis;
	}

	public void setTapis(boolean tapis) {
		this.tapis = tapis;
	}

	public boolean isCartes_visibles() {
		return cartesVisibles;
	}

	public void setCartes_visibles(boolean cartes_visibles) {
		this.cartesVisibles = cartes_visibles;
	}

	public boolean isRobot() {
		return robot;
	}

	public void setRobot(boolean robot) {
		this.robot = robot;
	}

	public int getNumero() {
		return numero;
	}

	public void setNumero(int numero) {
		this.numero = numero;
	}

	public Combinaison getCombinaison() {
		return combinaison;
	}

	public void setCombinaison(Combinaison combinaison) {
		this.combinaison = combinaison;
	}


	
}
