package fr.cnam.poker.robot;

import java.util.ArrayList;

import fr.cnam.poker.commun.Carte;
import fr.cnam.poker.commun.Combinaison;
import fr.cnam.poker.commun.Joueur;
import fr.cnam.poker.serveur.ModeleJeuPoker;

public class RobotImpl implements Robot {

	private Joueur joueur;
	private int nbSurEnchere;

	public RobotImpl(Joueur joueur) {
		this.joueur = joueur;
		this.nbSurEnchere=0;
	}

	public Joueur getJoueur() {
		return joueur;
	}

	// Le robot ne peux jamais tre donneur (control parle MODELE de Poker)
	// 
	// On implemente un fonctionnement minimal de comportement :
	//   - si aucun joueur n'ont encore miss alors le robot mise la valeur de la double blind.
	//   - si tous les joueurs qui n'ont pas abandonns ont tous miss la mme somme alors il checke (passe)
	//   sinon il mise des jetons pour s'aligner  la valeur max des mises
	//   - si tous les joeurs ont abandonns (sauf le robot lui-mme et sauf les autres robots) alors il rammasse le pot
	//     Ainsi pour que le robot puisse ramasser le pot, les joueurs doivent abandonner.
	//   - si au moins 1 joueur a fait tapis alors le robot fait tapis
	//
	public void jouer() {
		if (isTousAbandonnes()) {
			ModeleJeuPoker.getInstance().actionJoueur(joueur.getNumero(), "RAMASSER_POT");
			return;
		}
		else if (isTapis()) {
			ModeleJeuPoker.getInstance().actionJoueur(joueur.getNumero(), "FAIRE_TAPIS");
			return;
		}
		else if (isMisesZero()) {
			int doubeBlind = ModeleJeuPoker.getInstance().getTablePoker().getBlind()*2;
			ModeleJeuPoker.getInstance().actionJoueur(joueur.getNumero(), "MISER",""+doubeBlind);
			return;
		}
		else if (isMisesEgales()) {
			ModeleJeuPoker.getInstance().actionJoueur(joueur.getNumero(), "CHECKER");
			return;
		}
		else {
			int misesAdversaires= maxMises();
			int misesRobot = joueur.getNbJetonsMises();
			int aMiser=misesAdversaires-misesRobot;
			ModeleJeuPoker.getInstance().actionJoueur(joueur.getNumero(), "MISER",""+aMiser);
		}
	}

	// Retourne la valeur de la plus grosses mise mise par les joueurs
	//
	public int maxMises() {
		int max=0;
		for(Joueur joueur:ModeleJeuPoker.getInstance().getTablePoker().getJoueurs()) {
			if (!joueur.isAbandon() && joueur.getNbJetonsMises()>max)
				max=joueur.getNbJetonsMises();
		}
		return max;
	}

	// Si tous les joueurs ont abandonns
	//
	public boolean isTousAbandonnes() {
		for(Joueur joueur:ModeleJeuPoker.getInstance().getTablePoker().getJoueurs())
			if ( (!joueur.isAbandon()) && (joueur!=this.joueur) && (!joueur.isRobot())) return false;
		return true;
	}

	// si tous les joueurs ont miss la mme somme
	//
	public boolean isMisesEgales() {
		int val=-1;
		for(Joueur joueur:ModeleJeuPoker.getInstance().getTablePoker().getJoueurs()) {
			if (!joueur.isAbandon() )
				if (val==-1) val=joueur.getNbJetonsMises();
				else if (joueur.getNbJetonsMises()!=val) return false;
		}
		return true;
	}

	// Si au moins un joueur a fait tapis
	//
	public boolean isTapis() {
		for(Joueur joueur:ModeleJeuPoker.getInstance().getTablePoker().getJoueurs())
			if (joueur.isTapis()) return true;
		return false;
	}

	// si les joueurs n'ont pas encore miss
	//
	public boolean isMisesZero() {
		for(Joueur joueur:ModeleJeuPoker.getInstance().getTablePoker().getJoueurs()) {
			if (!joueur.isAbandon() )
				if (joueur.getNbJetonsMises()!=0) return false;
		}
		return true;
	}


}
