package fr.cnam.poker.serveur;

import java.io.*;
import java.net.*;

import fr.cnam.poker.commun.*;

// Classe de dfinition du controleur.
// Ce controleur permet de recevoir les requetes du client (les IHM de joueurs) afin
//  de transformer ces requetes en appel de mthode au MODELE (ModeleJeuPoker)
//
// Ce controleur est un SINGLETON
//
public class ControleurDistantPoker {
	
	// Dfinition du singleton
	private static ControleurDistantPoker singleton=null;
	public static ControleurDistantPoker getInstance() throws Exception {
		if (singleton==null)singleton=new ControleurDistantPoker();
		return singleton;
	}
	
	private ControleurDistantPoker() {}
	
	// Dmarrer les erveur de socket
	//
	public void start(int portServeurSocket) throws Exception {
		ModeleJeuPoker jeu = ModeleJeuPoker.getInstance();       // Rcupration du MODLE
		ServerSocket sos = new ServerSocket(portServeurSocket);  // Cration du serveur de scoket
		while(true)
		{
			Socket soc = sos.accept();
			OutputStream os=soc.getOutputStream();
			ObjectOutputStream oos=new ObjectOutputStream(os);
			InputStream is=soc.getInputStream();
			ObjectInputStream ois=new ObjectInputStream(is);


			Object requete = ois.readObject();  // Lecture de la requete reue
			
			System.out.println("REQUETE RECUE : "+requete.getClass().getName());

			// Si la requete recue est la demande d'abonnement d'une des IHM
			//
			if (requete instanceof RequeteAbonnement) {
				RequeteAbonnement reqAbonnement = (RequeteAbonnement)requete;	
				
				// Appel  la mthode d'abonnement
				int numeroJoueur = ModeleJeuPoker.getInstance().sabonner(reqAbonnement.getHost(), 
																	     reqAbonnement.getPort(),
																	     reqAbonnement.getNomJoueur());
				// Envoi de la rponse  l'abonnnement (qui contient le numero du joueur)
				oos.writeObject(new ReponseRequeteAbonnement(numeroJoueur));
				ModeleJeuPoker.getInstance().getObservable().notifier(ModeleJeuPoker.getInstance().getTablePoker());
			}
			
			// Si la requete est la ralisation d'un action d'un des jouueurs (dans l'IHM du joueur)
			//
			if (requete instanceof RequeteAction)
			{
				RequeteAction req = (RequeteAction)requete;
				
				// Appel  la mthode d'action du joueur
				//     
				ModeleJeuPoker.getInstance().actionJoueur(req.getNumeroJoueur(),
						                                  req.getNomAction(),
						                                  req.getParametres());
				// Envoi de la rponse vide (toujours envoyer une rponse)
				oos.writeObject(new ReponseVoid());
			}

		}
	}

}
