package fr.cnam.poker.serveur;
import java.util.ArrayList;
import java.util.Random;
import fr.cnam.poker.commun.Carte;

//Classe de dfinition du sabot de cartes qui permet de distribuer les cartes
//Les cartes distribues sont mmorises dans le sabot afin de pouvoir les
//remettre  la fin du sabot.
//
public class Sabot {
	private ArrayList<Carte> cartes;           // Les cartes restantes du sabot
	private ArrayList<Carte> cartesSorties;   // Les cartes retires du sabot

	// Constructeur
	//  Cration de toutes les cartes de 52 cartes
	//
	public Sabot() {
		cartes=new ArrayList<Carte>();
		cartesSorties=new ArrayList<Carte>();
		cartes.add(new Carte(Carte.TREFLE,"2"));
		cartes.add(new Carte(Carte.TREFLE,"3"));
		cartes.add(new Carte(Carte.TREFLE,"4"));
		cartes.add(new Carte(Carte.TREFLE,"5"));
		cartes.add(new Carte(Carte.TREFLE,"6"));
		cartes.add(new Carte(Carte.TREFLE,"7"));
		cartes.add(new Carte(Carte.TREFLE,"8"));
		cartes.add(new Carte(Carte.TREFLE,"9"));
		cartes.add(new Carte(Carte.TREFLE,"10"));
		cartes.add(new Carte(Carte.TREFLE,Carte.VALET));
		cartes.add(new Carte(Carte.TREFLE,Carte.DAME));
		cartes.add(new Carte(Carte.TREFLE,Carte.ROI));
		cartes.add(new Carte(Carte.TREFLE,Carte.AS));

		cartes.add(new Carte(Carte.CARREAU,"2"));
		cartes.add(new Carte(Carte.CARREAU,"3"));
		cartes.add(new Carte(Carte.CARREAU,"4"));
		cartes.add(new Carte(Carte.CARREAU,"5"));
		cartes.add(new Carte(Carte.CARREAU,"6"));
		cartes.add(new Carte(Carte.CARREAU,"7"));
		cartes.add(new Carte(Carte.CARREAU,"8"));
		cartes.add(new Carte(Carte.CARREAU,"9"));
		cartes.add(new Carte(Carte.CARREAU,"10"));
		cartes.add(new Carte(Carte.CARREAU,Carte.VALET));
		cartes.add(new Carte(Carte.CARREAU,Carte.DAME));
		cartes.add(new Carte(Carte.CARREAU,Carte.ROI));
		cartes.add(new Carte(Carte.CARREAU,Carte.AS));

		cartes.add(new Carte(Carte.COEUR,"2"));
		cartes.add(new Carte(Carte.COEUR,"3"));
		cartes.add(new Carte(Carte.COEUR,"4"));
		cartes.add(new Carte(Carte.COEUR,"5"));
		cartes.add(new Carte(Carte.COEUR,"6"));
		cartes.add(new Carte(Carte.COEUR,"7"));
		cartes.add(new Carte(Carte.COEUR,"8"));
		cartes.add(new Carte(Carte.COEUR,"9"));
		cartes.add(new Carte(Carte.COEUR,"10"));
		cartes.add(new Carte(Carte.COEUR,Carte.VALET));
		cartes.add(new Carte(Carte.COEUR,Carte.DAME));
		cartes.add(new Carte(Carte.COEUR,Carte.ROI));
		cartes.add(new Carte(Carte.COEUR,Carte.AS));

		cartes.add(new Carte(Carte.PIQUE,"2"));
		cartes.add(new Carte(Carte.PIQUE,"3"));
		cartes.add(new Carte(Carte.PIQUE,"4"));
		cartes.add(new Carte(Carte.PIQUE,"5"));
		cartes.add(new Carte(Carte.PIQUE,"6"));
		cartes.add(new Carte(Carte.PIQUE,"7"));
		cartes.add(new Carte(Carte.PIQUE,"8"));
		cartes.add(new Carte(Carte.PIQUE,"9"));
		cartes.add(new Carte(Carte.PIQUE,"10"));
		cartes.add(new Carte(Carte.PIQUE,Carte.VALET));
		cartes.add(new Carte(Carte.PIQUE,Carte.DAME));
		cartes.add(new Carte(Carte.PIQUE,Carte.ROI));
		cartes.add(new Carte(Carte.PIQUE,Carte.AS));
	}

	// Mthode qui permet de prendre la 1re carte du sabot
	//   Cette carte est retire de la liste des cartes du sabot
	//    et ajoute dans la liste des cartes sorties du sabot
	// Retourne la carte prise
	//
	public Carte prendreCarte() {
		if (cartes.size()==0) remettreCartes();  // Au cas o le sabot soit vide on remet les cartes sorties
		Carte carte = cartes.remove(0);
		cartesSorties.add(carte);
		return carte;
	}

	// Mthode qui remet dans le sabot toutes les cartes qui ont t sorties
	//
	public void remettreCartes() {
		melangerCartesSorties();  // On mlange les cartes sorties avant de les remettre
		for(Carte c:cartesSorties) cartes.add(c);
	}

	// Mthode qui mlange les cartes restantes du sabot
	//   Echange alatoirement 1000 fois deux cartes du sabot
	//
	public void melanger() {
		Random rdm= new Random(System.currentTimeMillis());
		for(int i=0;i<1000;i++) {
			int k1=rdm.nextInt(cartes.size());
			int k2=rdm.nextInt(cartes.size());
			Carte tmp=cartes.get(k1);
			cartes.set(k1, cartes.get(k2));
			cartes.set(k2,tmp);
		}
	}

	// Mthode qui permet de couper le jeu de carte alatoirement 
	//
	public void couper() {
		Random rdm= new Random(System.currentTimeMillis());
		ArrayList<Carte> cartesCoupe =new ArrayList<Carte>();
		int k=rdm.nextInt(cartes.size());
		for(int i=0;i<k;i++) {
			Carte carte = cartes.remove(0);
			cartesCoupe.add(carte);
		}
		for(Carte c:cartesCoupe) cartes.add(c);
	}

	// Mthode qui mlange les cartes restantes du sabot
	//   Echange alatoirement 1000 fois deux cartes des cartes sorties
	//
	private void melangerCartesSorties() {
		if (cartesSorties.size()<2) return;
		Random rdm= new Random(System.currentTimeMillis());
		for(int i=0;i<1000;i++) {
			int k1=rdm.nextInt(cartesSorties.size());
			int k2=rdm.nextInt(cartesSorties.size());
			Carte tmp=cartesSorties.get(k1);
			cartesSorties.set(k1, cartesSorties.get(k2));
			cartesSorties.set(k2,tmp);
		}
	}
}
